<?php

namespace Shopping\Shipping\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Shopping\Shipping\Http\Requests\CountryRequest;
use Shopping\Shipping\Interfaces\CountryRepositoryInterface;
use Shopping\Shipping\Models\Country;

/**
 * Resource controller class for country.
 */
class CountryResourceController extends BaseController
{

    /**
     * Initialize country resource controller.
     *
     * @param type CountryRepositoryInterface $country
     *
     * @return null
     */
    public function __construct(CountryRepositoryInterface $country)
    {
        parent::__construct();
        $this->repository = $country;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Shopping\Shipping\Repositories\Criteria\CountryResourceCriteria::class);
    }

    /**
     * Display a list of country.
     *
     * @return Response
     */
    public function index(CountryRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Shopping\Shipping\Repositories\Presenter\CountryPresenter::class)
                ->$function();
        }

        $countries = $this->repository->paginate();

        return $this->response->title(trans('shipping::country.names'))
            ->view('shipping::country.index', true)
            ->data(compact('countries'))
            ->output();
    }

    /**
     * Display country.
     *
     * @param Request $request
     * @param Model   $country
     *
     * @return Response
     */
    public function show(CountryRequest $request, Country $country)
    {

        if ($country->exists) {
            $view = 'shipping::country.show';
        } else {
            $view = 'shipping::country.new';
        }

        return $this->response->title(trans('app.view') . ' ' . trans('shipping::country.name'))
            ->data(compact('country'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new country.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(CountryRequest $request)
    {

        $country = $this->repository->newInstance([]);
        return $this->response->title(trans('app.new') . ' ' . trans('shipping::country.name')) 
            ->view('shipping::country.create', true) 
            ->data(compact('country'))
            ->output();
    }

    /**
     * Create new country.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(CountryRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $country                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('shipping::country.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('shipping/country/' . $country->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/shipping/country'))
                ->redirect();
        }

    }

    /**
     * Show country for editing.
     *
     * @param Request $request
     * @param Model   $country
     *
     * @return Response
     */
    public function edit(CountryRequest $request, Country $country)
    {
        return $this->response->title(trans('app.edit') . ' ' . trans('shipping::country.name'))
            ->view('shipping::country.edit', true)
            ->data(compact('country'))
            ->output();
    }

    /**
     * Update the country.
     *
     * @param Request $request
     * @param Model   $country
     *
     * @return Response
     */
    public function update(CountryRequest $request, Country $country)
    {
        try {
            $attributes = $request->all();

            $country->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('shipping::country.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('shipping/country/' . $country->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('shipping/country/' . $country->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the country.
     *
     * @param Model   $country
     *
     * @return Response
     */
    public function destroy(CountryRequest $request, Country $country)
    {
        try {

            $country->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('shipping::country.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('shipping/country/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('shipping/country/' . $country->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple country.
     *
     * @param Model   $country
     *
     * @return Response
     */
    public function delete(CountryRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('shipping::country.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('shipping/country'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/shipping/country'))
                ->redirect();
        }

    }

    /**
     * Restore deleted countries.
     *
     * @param Model   $country
     *
     * @return Response
     */
    public function restore(CountryRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('shipping::country.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/shipping/country'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/shipping/country/'))
                ->redirect();
        }

    }

}
